from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 币安码
class _GiftCardEndpoints():
    set_createCode = ['https://api.binance.com/','POST', '/sapi/v1/giftcard/createCode', True]  # 创建币安码 (USER_DATA)
    set_redeemCode = ['https://api.binance.com/','POST', '/sapi/v1/giftcard/redeemCode', True]  # 兑现币安码 (USER_DATA)
    get_verify = ['https://api.binance.com/','GET', '/sapi/v1/giftcard/verify', True]  # 验证币安码 (USER_DATA)
    get_cryptography_rsa_public_key = ['https://api.binance.com/','GET', '/sapi/v1/giftcard/cryptography/rsa-public-key', True]  # 获取RSA Public Key (USER_DATA)
    set_buyCode = ['https://api.binance.com/','POST', '/sapi/v1/giftcard/buyCode', True]  # 购买币安码 (TRADE)
    get_buyCode_token_limit = ['https://api.binance.com/','GET', '/sapi/v1/giftcard/buyCode/token-limit', True]  # 获取货币使用限制 (USER_DATA)


class GiftCard(Client):
    endpoints = _GiftCardEndpoints

    # 创建币安码 (USER_DATA)
    def set_createCode(self, token: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-118

        Name      	Type  	Mandatory	Description
        token     	str   	YES      	币安码中的数字货币币种
        amount    	DOUBLE	YES      	币安码中的数字货币数量
        recvWindow	int   	NO
        '''
        return self.send_request(*self.endpoints.set_createCode, **to_local(locals()))

    # 兑现币安码 (USER_DATA)
    def set_redeemCode(self, code: str = '', externalUid: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-119

        Name       	Type	Mandatory	Description
        code       	str 	YES      	用于赎回的binance code，支持加密&未加密两种方式
        externalUid	str 	NO       	每个外部用户 ID 代表合作伙伴平台上的某个用户。该功能帮助您识别不同用户的兑现行为，例如兑现频次和金额。它还有助于对单个账户进行风险和限额控制，例如设置单个账户每日兑现金额、频次和卡密输错次数的上限。这也将防止单个帐户突破合作伙伴的每日兑现限额从而导致合作伙伴的账户在当日无法继续制码或者兑现。如果您有外部的网站且有不同的用户在您的平台上兑现 Binance Code或礼品卡，我们强烈建议您使用此功能并将您用户的用户 ID 传输给我们来进行风控。为保护用户的信息安全，您可以选择以任何格式（上限为 400 个字符）传输用户 ID。
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.set_redeemCode, **to_local(locals()))

    # 验证币安码 (USER_DATA)
    def get_verify(self, referenceNo: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-120

        Name       	Type	Mandatory	Description
        referenceNo	str 	YES      	参考号
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_verify, **to_local(locals()))

    # 获取RSA Public Key (USER_DATA)
    def get_cryptography_rsa_public_key(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#rsa-public-key-user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_cryptography_rsa_public_key, **to_local(locals()))

    # 购买币安码 (TRADE)
    def set_buyCode(self, baseToken: str = '', faceToken: str = '', baseTokenAmount: Union[float, int] = '',
                    recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-24

        Name           	Type  	Mandatory	Description
        baseToken      	str   	YES      	你用来支付的货币，例如：BUSD
        faceToken      	str   	YES      	你购买的礼品卡面额，例如：BNB。如果 faceToken = baseToken，将等同于使用 createCode API
        baseTokenAmount	DOUBLE	YES      	支付的货币数量，例如：1.002
        recvWindow     	int   	NO
        '''
        return self.send_request(*self.endpoints.set_buyCode, **to_local(locals()))

    # 获取货币使用限制 (USER_DATA)
    def get_buyCode_token_limit(self, baseToken: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-121

        Name      	Type	Mandatory	Description
        baseToken 	str 	YES      	你用来支付的货币，例如：BUSD
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_buyCode_token_limit, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 合约
class _FuturesEndpoints():
    set_transfer = ['https://api.binance.com/','POST', '/sapi/v1/futures/transfer', True]  # 合约资金划转 (USER_DATA)
    get_transfer = ['https://api.binance.com/','GET', '/sapi/v1/futures/transfer', True]  # 获取合约资金划转历史 (USER_DATA)
    get_loan_borrow_history = ['https://api.binance.com/','GET', '/sapi/v1/futures/loan/borrow/history', True]  # 混合保证金借款历史 (USER_DATA)
    get_loan_repay_history = ['https://api.binance.com/','GET', '/sapi/v1/futures/loan/repay/history', True]  # 混合保证金还款历史 (USER_DATA)
    get_loan_wallet = ['https://api.binance.com/','GET', '/sapi/v2/futures/loan/wallet', True]  # 混合保证金钱包V2 (USER_DATA)
    get_loan_adjustCollateral_history = ['https://api.binance.com/','GET', '/sapi/v1/futures/loan/adjustCollateral/history', True]  # 混合保证金调整质押率历史 (USER_DATA)
    get_loan_liquidationHistory = ['https://api.binance.com/','GET', '/sapi/v1/futures/loan/liquidationHistory', True]  # 混合保证金强平历史 (USER_DATA)
    get_loan_interestHistory = ['https://api.binance.com/','GET', '/sapi/v1/futures/loan/interestHistory', True]  # 混合保证金利息收取历史 (USER_DATA)


class Future(Client):
    endpoints = _FuturesEndpoints

    # 合约资金划转 (USER_DATA)
    def set_transfer(self, asset: str = '', amount: Union[float, int] = '', type: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-76

        Name      	Type            	Mandatory	Description
        asset     	str             	YES      	The asset being transferred, e.g., USDT
        amount    	Union[float,int]	YES      	The amount to be transferred
        type      	int             	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_transfer, **to_local(locals()))

    # 获取合约资金划转历史 (USER_DATA)
    def get_transfer(self, asset: str = '', startTime: int = '', endTime: int = '', current: int = '', size: int = '',
                     recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-77

        Name      	Type	Mandatory	Description
        asset     	str 	YES
        startTime 	int 	YES
        endTime   	int 	NO
        current   	int 	NO       	当前页面. 起始计数为1. 默认值1
        size      	int 	NO       	单页数据条目数，默认:10 最大:100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_transfer, **to_local(locals()))

    # 混合保证金借款历史 (USER_DATA)
    def get_loan_borrow_history(self, coin: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                                recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-78

        Name      	Type	Mandatory	Description
        coin      	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	default 500, max 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_borrow_history, **to_local(locals()))

    # 混合保证金还款历史 (USER_DATA)
    def get_loan_repay_history(self, coin: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                               recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-79

        Name      	Type	Mandatory	Description
        coin      	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	default 500, max 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_repay_history, **to_local(locals()))

    # 混合保证金钱包V2 (USER_DATA)
    def get_loan_wallet(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#v2-user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_wallet, **to_local(locals()))

    # 混合保证金调整质押率历史 (USER_DATA)
    def get_loan_adjustCollateral_history(self, loanCoin: str = '', collateralCoin: str = '', startTime: int = '',
                                          endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-80

        Name          	Type	Mandatory	Description
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        limit         	int 	NO       	default 500, max 1000
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_adjustCollateral_history, **to_local(locals()))

    # 混合保证金强平历史 (USER_DATA)
    def get_loan_liquidationHistory(self, loanCoin: str = '', collateralCoin: str = '', startTime: int = '',
                                    endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-81

        Name          	Type	Mandatory	Description
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        limit         	int 	NO       	default 500, max 1000
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_liquidationHistory, **to_local(locals()))

    # 混合保证金利息收取历史 (USER_DATA)
    def get_loan_interestHistory(self, collateralCoin: str = '', startTime: int = '', endTime: int = '',
                                 current: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-82

        Name          	Type	Mandatory	Description
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页。 开始值 1。 默认:1
        limit         	int 	NO       	默认 500，最大 1000
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan_interestHistory, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 币安宝
class _SavingsEndpoints():
    get_daily_product_list = ['https://api.binance.com/','GET', '/sapi/v1/lending/daily/product/list', True]  # 获取活期产品列表 (USER_DATA)
    get_daily_userLeftQuota = ['https://api.binance.com/','GET', '/sapi/v1/lending/daily/userLeftQuota', True]  # 获取用户当日剩余活期可申购余额 (USER_DATA)
    set_daily_purchase = ['https://api.binance.com/','POST', '/sapi/v1/lending/daily/purchase', True]  # 申购活期产品 (USER_DATA)
    get_daily_userRedemptionQuota = ['https://api.binance.com/','GET', '/sapi/v1/lending/daily/userRedemptionQuota', True]  # 获取用户当日活期可赎回余额 (USER_DATA)
    set_daily_redeem = ['https://api.binance.com/','POST', '/sapi/v1/lending/daily/redeem', True]  # 赎回活期产品 (USER_DATA)
    get_daily_token_position = ['https://api.binance.com/','GET', '/sapi/v1/lending/daily/token/position', True]  # 用户活期产品持仓 (USER_DATA)
    get_project_list = ['https://api.binance.com/','GET', '/sapi/v1/lending/project/list', True]  # 查询定期/活动产品列表 (USER_DATA)
    set_customizedFixed_purchase = ['https://api.binance.com/','POST', '/sapi/v1/lending/customizedFixed/purchase', True]  # 申购定期/活动产品 (USER_DATA)
    get_project_position_list = ['https://api.binance.com/','GET', '/sapi/v1/lending/project/position/list', True]  # 用户定期/活动持仓 (USER_DATA)
    get_union_account = ['https://api.binance.com/','GET', '/sapi/v1/lending/union/account', True]  # 币安宝账户信息 (USER_DATA)
    get_union_purchaseRecord = ['https://api.binance.com/','GET', '/sapi/v1/lending/union/purchaseRecord', True]  # 获取申购记录 (USER_DATA)
    get_union_redemptionRecord = ['https://api.binance.com/','GET', '/sapi/v1/lending/union/redemptionRecord', True]  # 获取赎回记录 (USER_DATA)
    get_union_interestHistory = ['https://api.binance.com/','GET', '/sapi/v1/lending/union/interestHistory', True]  # 获取利息历史 (USER_DATA)
    set_positionChanged = ['https://api.binance.com/','POST', '/sapi/v1/lending/positionChanged', True]  # 定期/活动持仓转活期持仓 (USER_DATA)


class Savings(Client):
    endpoints = _SavingsEndpoints

    # 获取活期产品列表 (USER_DATA)
    def get_daily_product_list(self, status: str = '', featured: str = '', current: int = '', size: int = '',
                               recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-46

        Name      	Type	Mandatory	Description
        status    	str 	NO       	"ALL", "SUBSCRIBABLE", "UNSUBSCRIBABLE";  默认: "ALL"
        featured  	str 	NO       	"ALL", "TRUE";  默认: "ALL"
        current   	int 	NO       	当前页面. 默认: 1, 最小: 1
        size      	int 	NO       	默认: 50, 最大: 100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_daily_product_list, **to_local(locals()))

    # 获取用户当日剩余活期可申购余额 (USER_DATA)
    def get_daily_userLeftQuota(self, productId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-47

        Name      	Type	Mandatory	Description
        productId 	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_daily_userLeftQuota, **to_local(locals()))

    # 申购活期产品 (USER_DATA)
    def set_daily_purchase(self, productId: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-48

        Name      	Type            	Mandatory	Description
        productId 	str             	YES
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_daily_purchase, **to_local(locals()))

    # 获取用户当日活期可赎回余额 (USER_DATA)
    def get_daily_userRedemptionQuota(self, productId: str = '', type: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-49

        Name      	Type	Mandatory	Description
        productId 	str 	YES
        type      	str 	YES      	"FAST", "NORMAL"
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_daily_userRedemptionQuota, **to_local(locals()))

    # 赎回活期产品 (USER_DATA)
    def set_daily_redeem(self, productId: str = '', amount: Union[float, int] = '', type: str = '',
                         recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-50

        Name      	Type            	Mandatory	Description
        productId 	str             	YES
        amount    	Union[float,int]	YES
        type      	str             	YES      	"FAST", "NORMAL"
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_daily_redeem, **to_local(locals()))

    # 用户活期产品持仓 (USER_DATA)
    def get_daily_token_position(self, asset: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-51

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_daily_token_position, **to_local(locals()))

    # 查询定期/活动产品列表 (USER_DATA)
    def get_project_list(self, asset: str = '', type: str = '', status: str = '', isSortAsc: bool = '',
                         sortBy: str = '',
                         current: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-52

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        type      	str 	YES      	"CUSTOMIZED_FIXED"定期 , "ACTIVITY" 活动
        status    	str 	NO       	"ALL", "SUBSCRIBABLE", "UNSUBSCRIBABLE"; 默认 "ALL"
        isSortAsc 	bool	NO       	默认 "true"
        sortBy    	str 	NO       	"START_TIME", "LOT_SIZE", "INTEREST_RATE", "DURATION"; 默认 "START_TIME"
        current   	int 	NO       	分页页码. 默认:1
        size      	int 	NO       	单页显示条数，默认:10 最大:100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_project_list, **to_local(locals()))

    # 申购定期/活动产品 (USER_DATA)
    def set_customizedFixed_purchase(self, projectId: str = '', lot: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-53

        Name      	Type	Mandatory	Description
        projectId 	str 	YES
        lot       	int 	YES      	申购手数
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_customizedFixed_purchase, **to_local(locals()))

    # 用户定期/活动持仓 (USER_DATA)
    def get_project_position_list(self, asset: str = '', projectId: str = '', status: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-54

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        projectId 	str 	NO
        status    	str 	NO       	"HOLDING", "REDEEMED"
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_project_position_list, **to_local(locals()))

    # 币安宝账户信息 (USER_DATA)
    def get_union_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-55

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_union_account, **to_local(locals()))

    # 获取申购记录 (USER_DATA)
    def get_union_purchaseRecord(self, lendingType: str = '', asset: str = '', startTime: int = '', endTime: int = '',
                                 current: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-56

        Name       	Type	Mandatory	Description
        lendingType	str 	YES      	"DAILY" 表示活期, "ACTIVITI" 表示活动, "CUSTOMIZED_FIXED" 表示定期
        asset      	str 	NO
        startTime  	int 	NO
        endTime    	int 	NO
        current    	int 	NO       	Currently querying page. Start from 1. Default:1
        size       	int 	NO       	Default:10, Max:100
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_union_purchaseRecord, **to_local(locals()))

    # 获取赎回记录 (USER_DATA)
    def get_union_redemptionRecord(self, lendingType: str = '', asset: str = '', startTime: int = '', endTime: int = '',
                                   current: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-57

        Name       	Type	Mandatory	Description
        lendingType	str 	YES      	"DAILY" 表示活期, "ACTIVITY" 表示活动, "CUSTOMIZED_FIXED" 表示定期
        asset      	str 	NO
        startTime  	int 	NO
        endTime    	int 	NO
        current    	int 	NO       	Currently querying page. Start from 1. Default:1
        size       	int 	NO       	Default:10, Max:100
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_union_redemptionRecord, **to_local(locals()))

    # 获取利息历史 (USER_DATA)
    def get_union_interestHistory(self, lendingType: str = '', asset: str = '', startTime: int = '', endTime: int = '',
                                  current: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-58

        Name       	Type	Mandatory	Description
        lendingType	str 	YES      	"DAILY" 表示活期, "ACTIVITY" 表示活动, "CUSTOMIZED_FIXED" 表示定期
        asset      	str 	NO
        startTime  	int 	NO
        endTime    	int 	NO
        current    	int 	NO       	Currently querying page. Start from 1. Default:1
        size       	int 	NO       	Default:10, Max:100
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_union_interestHistory, **to_local(locals()))

    # 定期/活动持仓转活期持仓 (USER_DATA)
    def set_positionChanged(self, projectId: str = '', lot: int = '', positionId: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-59

        Name      	Type	Mandatory	Description
        projectId 	str 	YES
        lot       	int 	YES
        positionId	int 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_positionChanged, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 闪兑
class _ConvertEndpoints():
    get_exchangeInfo = ['https://api.binance.com/','GET', '/sapi/v1/convert/exchangeInfo', False]  # 查询可交易币对信息 (USER_DATA)
    get_assetInfo = ['https://api.binance.com/','GET', '/sapi/v1/convert/assetInfo', False]  # 查询可交易币种精度 (USER_DATA)
    set_getQuote = ['https://api.binance.com/','POST', '/sapi/v1/convert/getQuote', False]  # 发送获取报价请求 (USER_DATA)
    set_acceptQuote = ['https://api.binance.com/','POST', '/sapi/v1/convert/acceptQuote', False]  # 接受报价 (TRADE)
    get_orderStatus = ['https://api.binance.com/','GET', '/sapi/v1/convert/orderStatus', False]  # 查询订单状态 (USER_DATA)
    get_tradeFlow = ['https://api.binance.com/','GET', '/sapi/v1/convert/tradeFlow', True]  # 获取闪兑交易记录 (USER_DATA)


class Convert(Client):
    endpoints = _ConvertEndpoints

    # 查询可交易币对信息 (USER_DATA)
    def get_exchangeInfo(self, fromAsset: str = '', toAsset: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-112

        Name     	Type	Mandatory     	Description
        fromAsset	str 	EITHER OR BOTH	用户售出币种
        toAsset  	str 	EITHER OR BOTH	用户买入币种
        '''
        return self.send_request(*self.endpoints.get_exchangeInfo, **to_local(locals()))

    # 查询可交易币种精度 (USER_DATA)
    def get_assetInfo(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-113

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO       	此值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_assetInfo, **to_local(locals()))

    # 发送获取报价请求 (USER_DATA)
    def set_getQuote(self, fromAsset: str = '', toAsset: str = '', fromAmount: Union[float, int] = '',
                     toAmount: Union[float, int] = '', validTime: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-114

        Name      	Type            	Mandatory	Description
        fromAsset 	str             	YES
        toAsset   	str             	YES
        fromAmount	Union[float,int]	EITHER   	这是成交后将被扣除的金额
        toAmount  	Union[float,int]	EITHER   	这是成交后将会获得的金额
        validTime 	str             	NO       	可以支持10s、30s、1m、2m等值，默认值为 10s
        recvWindow	int             	NO       	此值不能大于 60000
        '''
        return self.send_request(*self.endpoints.set_getQuote, **to_local(locals()))

    # 接受报价 (TRADE)
    def set_acceptQuote(self, quoteId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-23

        Name      	Type	Mandatory	Description
        quoteId   	str 	YES
        recvWindow	int 	NO       	此值不能大于 60000
        '''
        return self.send_request(*self.endpoints.set_acceptQuote, **to_local(locals()))

    # 查询订单状态 (USER_DATA)
    def get_orderStatus(self, orderId: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-115

        Name   	Type	Mandatory	Description
        orderId	str 	YES
        '''
        return self.send_request(*self.endpoints.get_orderStatus, **to_local(locals()))

    # 获取闪兑交易记录 (USER_DATA)
    def get_tradeFlow(self, startTime: int = '', endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-116

        Name      	Type	Mandatory	Description
        startTime 	int 	YES
        endTime   	int 	YES
        limit     	int 	NO       	默认 100, 最大 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_tradeFlow, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


class _MarginAccountTradeEndpoints():
    set_transfer = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/transfer', True]  # 全仓杠杆账户划转 (MARGIN)
    set_loan = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/loan', True]  # 杠杆账户借贷 (MARGIN)
    set_repay = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/repay', True]  # 杠杆账户归还借贷 (MARGIN)
    get_asset = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/asset', False]  # 查询杠杆资产 (MARKET_DATA)
    get_pair = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/pair', False]  # 查询全仓杠杆交易对 (MARKET_DATA)
    get_allAssets = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/allAssets', False]  # 获取所有杠杆资产信息 (MARKET_DATA)
    get_allPairs = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/allPairs', False]  # 获取所有全仓杠杆交易对(MARKET_DATA)
    get_priceIndex = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/priceIndex', False]  # 查询杠杆价格指数 (MARKET_DATA)
    set_order = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/order', True]  # 杠杆账户下单 (TRADE)
    cancel_order = ['https://api.binance.com/', 'DELETE', '/sapi/v1/margin/order', True]  # 杠杆账户撤销订单 (TRADE)
    cancel_openOrders = ['https://api.binance.com/', 'DELETE', '/sapi/v1/margin/openOrders', True]  # 杠杆账户撤销单一交易对的所有挂单 (TRADE)
    get_transfer = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/transfer', True]  # 获取全仓杠杆划转历史 (USER_DATA)
    get_loan = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/loan', True]  # 查询借贷记录 (USER_DATA)
    get_repay = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/repay', True]  # 查询还贷记录 (USER_DATA)
    get_interestHistory = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/interestHistory', True]  # 获取利息历史 (USER_DATA)
    get_cross_account = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/account', True]  # 查询全仓杠杆账户详情 (USER_DATA)
    get_forceLiquidationRec = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/forceLiquidationRec', True]  # 获取账户强制平仓记录(USER_DATA)
    get_order = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/order', True]  # 查询杠杆账户订单 (USER_DATA)
    get_openOrders = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/openOrders', True]  # 查询杠杆账户挂单记录 (USER_DATA)
    get_allOrders = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/allOrders', True]  # 查询杠杆账户的所有订单 (USER_DATA)
    set_order_oco = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/order/oco', True]  # 杠杆账户 OCO 下单(TRADE)
    cancel_orderList = ['https://api.binance.com/', 'DELETE', '/sapi/v1/margin/orderList', True]  # 取消杠杆账户 OCO 订单(TRADE)
    get_orderList = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/orderList', True]  # 查询杠杆账户 OCO (USER_DATA)
    get_allOrderList = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/allOrderList', True]  # 查询特定杠杆账户所有 OCO (USER_DATA)
    get_openOrderList = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/openOrderList', True]  # 查询杠杆账户 OCO 挂单 (USER_DATA)
    get_myTrades = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/myTrades', True]  # 查询杠杆账户交易历史 (USER_DATA)
    get_maxBorrowable = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/maxBorrowable', True]  # 查询账户最大可借贷额度(USER_DATA)
    get_maxTransferable = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/maxTransferable', True]  # 查询最大可转出额 (USER_DATA)
    get_tradeCoeff = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/tradeCoeff', True]  # 查询Margin账户信息汇总 (USER_DATA)
    set_isolated_transfer = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/isolated/transfer', True]  # 杠杆逐仓账户划转 (MARGIN)
    get_isolated_transfer = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolated/transfer', True]  # 获取杠杆逐仓划转历史 (USER_DATA)
    get_isolated_account = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolated/account', True]  # 查询杠杆逐仓账户信息 (USER_DATA)
    delete_isolated_account = ['https://api.binance.com/', 'DELETE', '/sapi/v1/margin/isolated/account', True]  # 杠杆逐仓账户停用 (TRADE)
    set_isolated_account = ['https://api.binance.com/', 'POST', '/sapi/v1/margin/isolated/account', True]  # 杠杆逐仓账户启用 (TRADE)
    get_isolated_accountLimit = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolated/accountLimit', True]  # 查询杠杆逐仓账户启用限制 (USER_DATA)
    get_isolated_pair = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolated/pair', True]  # 查询逐仓杠杆交易对 (USER_DATA)
    get_isolated_allPairs = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolated/allPairs', True]  # 获取所有逐仓杠杆交易对(USER_DATA)
    set_bnbBurn = ['https://api.binance.com/', 'POST', '/sapi/v1/bnbBurn', True]  # 现货交易和杠杆利息BNB抵扣开关(USER_DATA)
    get_bnbBurn = ['https://api.binance.com/', 'GET', '/sapi/v1/bnbBurn', True]  # 获取BNB抵扣开关状态 (USER_DATA)
    get_interestRateHistory = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/interestRateHistory', True]  # 获取杠杆利率历史 (USER_DATA)
    get_crossMarginData = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/crossMarginData', True]  # 获取全仓杠杆利率及限额 (USER_DATA)
    get_isolatedMarginData = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolatedMarginData', True]  # 获取逐仓杠杆利率及限额 (USER_DATA)
    get_isolatedMarginTier = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/isolatedMarginTier', True]  # 获取逐仓档位信息 (USER_DATA)
    get_rateLimit_order = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/rateLimit/order', False]  # 查询目前杠杆账户下单数 (TRADE)
    get_dribblet = ['https://api.binance.com/', 'GET', '/sapi/v1/margin/dribblet', True]  # 杠杆小额资产转换BNB历史 (USER_DATA


class MarginAccountTrade(Client):
    endpoints = _MarginAccountTradeEndpoints

    # 全仓杠杆账户划转 (MARGIN)
    def set_transfer(self, asset: str = '', amount: Union[float, int] = '', type: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-5

        Name      	Type            	Mandatory	Description
        asset     	str             	YES      	被划转的资产, 比如, BTC
        amount    	Union[float,int]	YES      	划转数量
        type      	int             	YES      	1: 主账户向全仓杠杆账户划转 2: 全仓杠杆账户向主账户划转
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_transfer, **to_local(locals()))

    # 杠杆账户借贷 (MARGIN)
    def set_loan(self, asset: str = '', isIsolated: str = '', symbol: str = '', amount: Union[float, int] = '',
                 recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-6

        Name      	Type            	Mandatory	Description
        asset     	str             	YES
        isIsolated	str             	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol    	str             	NO       	逐仓交易对，配合逐仓使用
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_loan, **to_local(locals()))

    # 杠杆账户归还借贷 (MARGIN)
    def set_repay(self, asset: str = '', isIsolated: str = '', symbol: str = '', amount: Union[float, int] = '',
                  recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-7

        Name      	Type            	Mandatory	Description
        asset     	str             	YES
        isIsolated	str             	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol    	str             	NO       	逐仓交易对，配合逐仓使用
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_repay, **to_local(locals()))

    # 查询杠杆资产 (MARKET_DATA)
    def get_asset(self, asset: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-2

        Name 	Type	Mandatory	Description
        asset	str 	YES
        '''
        return self.send_request(*self.endpoints.get_asset, **to_local(locals()))

    # 查询全仓杠杆交易对 (MARKET_DATA)
    def get_pair(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-3

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        '''
        return self.send_request(*self.endpoints.get_pair, **to_local(locals()))

    # 获取所有杠杆资产信息 (MARKET_DATA)
    def get_allAssets(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-4
        '''
        return self.send_request(*self.endpoints.get_allAssets, **to_local(locals()))

    # 获取所有全仓杠杆交易对(MARKET_DATA)
    def get_allPairs(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-5
        '''
        return self.send_request(*self.endpoints.get_allPairs, **to_local(locals()))

    # 查询杠杆价格指数 (MARKET_DATA)
    def get_priceIndex(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-6

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        '''
        return self.send_request(*self.endpoints.get_priceIndex, **to_local(locals()))

    # 杠杆账户下单 (TRADE)
    def set_order(self, symbol: str = '', isIsolated: str = '', side: str = '', type: str = '',
                  quantity: Union[float, int] = '', quoteOrderQty: Union[float, int] = '',
                  price: Union[float, int] = '',
                  stopPrice: Union[float, int] = '', newClientOrderId: str = '', icebergQty: Union[float, int] = '',
                  newOrderRespType: str = '', sideEffectType: str = '', timeInForce: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-8

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES
        isIsolated      	str             	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        side            	str             	YES
        type            	str             	YES      	详见枚举定义：订单类型
        quantity        	Union[float,int]	NO
        quoteOrderQty   	Union[float,int]	NO
        price           	Union[float,int]	NO
        stopPrice       	Union[float,int]	NO
        newClientOrderId	str             	NO       	客户自定义的唯一订单ID。若未发送自动生成。
        icebergQty      	Union[float,int]	NO
        newOrderRespType	str             	NO       	设置响应: JSON. ACK, RESULT, 或 FULL; MARKET 和 LIMIT 订单类型默认为 FULL, 所有其他订单默认为 ACK.
        sideEffectType  	str             	NO       	NO_SIDE_EFFECT, MARGIN_BUY, AUTO_REPAY;默认为 NO_SIDE_EFFECT.
        timeInForce     	str             	NO       	GTC,IOC,FOK
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order, **to_local(locals()))

    # 杠杆账户撤销订单 (TRADE)
    def cancel_order(self, symbol: str = '', isIsolated: str = '', orderId: int = '', origClientOrderId: str = '',
                     newClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-9

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        isIsolated       	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        orderId          	int 	NO
        origClientOrderId	str 	NO
        newClientOrderId 	str 	NO       	用于唯一识别此撤销订单，默认自动生成。
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_order, **to_local(locals()))

    # 杠杆账户撤销单一交易对的所有挂单 (TRADE)
    def cancel_openOrders(self, symbol: str = '', isIsolated: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-10

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_openOrders, **to_local(locals()))

    # 获取全仓杠杆划转历史 (USER_DATA)
    def get_transfer(self, asset: str = '', type: str = '', startTime: int = '', endTime: int = '', current: int = '',
                     size: int = '', archived: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-25

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        type      	str 	NO       	划转类型: ROLL_IN, ROLL_OUT
        startTime 	int 	NO
        endTime   	int 	NO
        current   	int 	NO       	当前查询页。 从 1开始。 默认:1
        size      	int 	NO       	默认:10 最大:100
        archived  	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_transfer, **to_local(locals()))

    # 查询借贷记录 (USER_DATA)
    def get_loan(self, asset: str = '', isolatedSymbol: str = '', txId: int = '', startTime: int = '',
                 endTime: int = '',
                 current: int = '', size: int = '', archived: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-26

        Name          	Type	Mandatory	Description
        asset         	str 	YES
        isolatedSymbol	str 	NO       	逐仓symbol
        txId          	int 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页。 开始值 1。 默认:1
        size          	int 	NO       	默认:10 最大:100
        archived      	str 	NO
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loan, **to_local(locals()))

    # 查询还贷记录 (USER_DATA)
    def get_repay(self, asset: str = '', isolatedSymbol: str = '', txId: int = '', startTime: int = '',
                  endTime: int = '',
                  current: int = '', size: int = '', archived: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-27

        Name          	Type	Mandatory	Description
        asset         	str 	YES
        isolatedSymbol	str 	NO       	逐仓symbol
        txId          	int 	NO       	返回 /sapi/v1/margin/repay
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页。开始值 1. 默认:1
        size          	int 	NO       	默认:10 最大:100
        archived      	str 	NO
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_repay, **to_local(locals()))

    # 获取利息历史 (USER_DATA)
    def get_interestHistory(self, asset: str = '', isolatedSymbol: str = '', startTime: int = '', endTime: int = '',
                            current: int = '', size: int = '', archived: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-28

        Name          	Type	Mandatory	Description
        asset         	str 	NO
        isolatedSymbol	str 	NO       	逐仓symbol
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页。 开始值 1. 默认:1
        size          	int 	NO       	默认:10 最大:100
        archived      	str 	NO
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_interestHistory, **to_local(locals()))

    # 获取账户强制平仓记录(USER_DATA)
    def get_forceLiquidationRec(self, startTime: int = '', endTime: int = '', isolatedSymbol: str = '',
                                current: int = '',
                                size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-29

        Name          	Type	Mandatory	Description
        startTime     	int 	NO
        endTime       	int 	NO
        isolatedSymbol	str 	NO
        current       	int 	NO       	当前查询页。 开始值 1. 默认:1
        size          	int 	NO       	默认:10 最大:100
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_forceLiquidationRec, **to_local(locals()))

    # 查询全仓杠杆账户详情 (USER_DATA)
    def get_cross_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-30

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_cross_account, **to_local(locals()))

    # 查询杠杆账户订单 (USER_DATA)
    def get_order(self, symbol: str = '', isIsolated: str = '', orderId: int = '', origClientOrderId: str = '',
                  recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-31

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        isIsolated       	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        orderId          	int 	NO
        origClientOrderId	str 	NO
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_order, **to_local(locals()))

    # 查询杠杆账户挂单记录 (USER_DATA)
    def get_openOrders(self, symbol: str = '', isIsolated: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-32

        Name      	Type	Mandatory	Description
        symbol    	str 	NO
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrders, **to_local(locals()))

    # 查询杠杆账户的所有订单 (USER_DATA)
    def get_allOrders(self, symbol: str = '', isIsolated: str = '', orderId: int = '', startTime: int = '',
                      endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-33

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        orderId   	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 500;最大500.
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_allOrders, **to_local(locals()))

    # 杠杆账户 OCO 下单(TRADE)
    def set_order_oco(self, symbol: str = '', isIsolated: str = '', listClientOrderId: str = '', side: str = '',
                      quantity: Union[float, int] = '', limitClientOrderId: str = '', price: Union[float, int] = '',
                      limitIcebergQty: Union[float, int] = '', stopClientOrderId: str = '',
                      stopPrice: Union[float, int] = '', stopLimitPrice: Union[float, int] = '',
                      stopIcebergQty: Union[float, int] = '', stopLimitTimeInForce: str = '',
                      newOrderRespType: str = '',
                      sideEffectType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-trade-3

        Name                	Type            	Mandatory	Description
        symbol              	str             	YES
        isIsolated          	str             	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        listClientOrderId   	str             	NO       	整个orderList的唯一ID
        side                	str             	YES      	详见枚举定义：订单方向
        quantity            	Union[float,int]	YES
        limitClientOrderId  	str             	NO       	限价单的唯一ID
        price               	Union[float,int]	YES
        limitIcebergQty     	Union[float,int]	NO
        stopClientOrderId   	str             	NO       	止损/止损限价单的唯一ID
        stopPrice           	Union[float,int]	YES
        stopLimitPrice      	Union[float,int]	NO
        stopIcebergQty      	Union[float,int]	NO
        stopLimitTimeInForce	str             	NO
        newOrderRespType    	str             	NO       	详见枚举定义：订单返回类型
        sideEffectType      	str             	NO       	NO_SIDE_EFFECT, MARGIN_BUY, AUTO_REPAY; 默认为 NO_SIDE_EFFECT
        recvWindow          	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order_oco, **to_local(locals()))

    # 取消杠杆账户 OCO 订单(TRADE)
    def cancel_orderList(self, symbol: str = '', isIsolated: str = '', orderListId: int = '',
                         listClientOrderId: str = '',
                         newClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-trade-4

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        isIsolated       	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        orderListId      	int 	NO
        listClientOrderId	str 	NO
        newClientOrderId 	str 	NO       	用户自定义的本次撤销操作的ID(注意不是被撤销的订单的自定义ID)。如无指定会自动赋值。
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_orderList, **to_local(locals()))

    # 查询杠杆账户 OCO (USER_DATA)
    def get_orderList(self, isIsolated: str = '', symbol: str = '', orderListId: int = '', origClientOrderId: str = '',
                      recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data-4

        Name             	Type	Mandatory	Description
        isIsolated       	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol           	str 	NO       	逐仓杠杆必填，全仓杠杆不支持该参数
        orderListId      	int 	NO
        origClientOrderId	str 	NO
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_orderList, **to_local(locals()))

    # 查询特定杠杆账户所有 OCO (USER_DATA)
    def get_allOrderList(self, isIsolated: str = '', symbol: str = '', fromId: int = '', startTime: int = '',
                         endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data-5

        Name      	Type	Mandatory	Description
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol    	str 	NO       	逐仓杠杆必填，全仓杠杆不支持该参数
        fromId    	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认值: 500; 最大值: 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_allOrderList, **to_local(locals()))

    # 查询杠杆账户 OCO 挂单 (USER_DATA)
    def get_openOrderList(self, isIsolated: str = '', symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data-6

        Name      	Type	Mandatory	Description
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol    	str 	NO       	逐仓杠杆必填，全仓杠杆不支持该参数
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrderList, **to_local(locals()))

    # 查询杠杆账户交易历史 (USER_DATA)
    def get_myTrades(self, symbol: str = '', isIsolated: str = '', orderId: int = '', startTime: int = '',
                     endTime: int = '', fromId: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-34

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        orderId   	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        fromId    	int 	NO       	获取TradeId，默认获取近期交易历史。
        limit     	int 	NO       	默认 500; 最大 1000.
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_myTrades, **to_local(locals()))

    # 查询账户最大可借贷额度(USER_DATA)
    def get_maxBorrowable(self, asset: str = '', isolatedSymbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-35

        Name          	Type	Mandatory	Description
        asset         	str 	YES
        isolatedSymbol	str 	NO       	逐仓交易对，适用于逐仓查询
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_maxBorrowable, **to_local(locals()))

    # 查询最大可转出额 (USER_DATA)
    def get_maxTransferable(self, asset: str = '', isolatedSymbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-36

        Name          	Type	Mandatory	Description
        asset         	str 	YES
        isolatedSymbol	str 	NO       	逐仓交易对，适用于逐仓查询
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_maxTransferable, **to_local(locals()))

    # 查询Margin账户信息汇总 (USER_DATA)
    def get_tradeCoeff(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_tradeCoeff, **to_local(locals()))

    # 杠杆逐仓账户划转 (MARGIN)
    def set_isolated_transfer(self, asset: str = '', symbol: str = '', transFrom: str = '', transTo: str = '',
                              amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-8

        Name      	Type            	Mandatory	Description
        asset     	str             	YES      	被划转的资产, 比如, BTC
        symbol    	str             	YES      	逐仓 symbol
        transFrom 	str             	YES      	"SPOT", "ISOLATED_MARGIN"
        transTo   	str             	YES      	"SPOT", "ISOLATED_MARGIN"
        amount    	Union[float,int]	YES      	划转数量
        recvWindow	int             	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.set_isolated_transfer, **to_local(locals()))

    # 获取杠杆逐仓划转历史 (USER_DATA)
    def get_isolated_transfer(self, asset: str = '', symbol: str = '', transFrom: str = '', transTo: str = '',
                              startTime: int = '', endTime: int = '', current: int = '', size: int = '',
                              archived: str = '',
                              recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-37

        Name      	Type	Mandatory       	Description
        asset     	str 	NO
        symbol    	str 	YES             	逐仓 symbol
        transFrom 	str 	NO              	"SPOT", "ISOLATED_MARGIN"
        transTo   	str 	NO              	"SPOT", "ISOLATED_MARGIN"
        startTime 	int 	NO
        endTime   	int 	NO
        current   	int 	NO              	当前查询页。 从 1开始。 默认:1
        size      	int 	NO              	默认:10 最大:100
        archived  	str 	NO
        recvWindow	int 	NO  赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_isolated_transfer, **to_local(locals()))

    # 查询杠杆逐仓账户信息 (USER_DATA)
    def get_isolated_account(self, symbols: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-38

        Name      	Type	Mandatory	Description
        symbols   	str 	NO       	最多可以传5个symbol; 由","分隔的字符串表示. e.g. "BTCUSDT,BNBUSDT,ADAUSDT"
        recvWindow	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_isolated_account, **to_local(locals()))

    # 杠杆逐仓账户停用 (TRADE)
    def delete_isolated_account(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-11

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.delete_isolated_account, **to_local(locals()))

    # 杠杆逐仓账户启用 (TRADE)
    def set_isolated_account(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-12

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_isolated_account, **to_local(locals()))

    # 查询杠杆逐仓账户启用限制 (USER_DATA)
    def get_isolated_accountLimit(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-39

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_isolated_accountLimit, **to_local(locals()))

    # 查询逐仓杠杆交易对 (USER_DATA)
    def get_isolated_pair(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-40

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        recvWindow	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_isolated_pair, **to_local(locals()))

    # 获取所有逐仓杠杆交易对(USER_DATA)
    def get_isolated_allPairs(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-41

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_isolated_allPairs, **to_local(locals()))

    # 现货交易和杠杆利息BNB抵扣开关(USER_DATA)
    def set_bnbBurn(self, spotBNBBurn: str = '', interestBNBBurn: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bnb-user_data-3

        Name           	Type	Mandatory	Description
        spotBNBBurn    	str 	NO       	"true" or "false", 是否使用BNB支付现货交易的手续费
        interestBNBBurn	str 	NO       	"true" or "false", 是否使用BNB支付杠杆贷款的利息
        recvWindow     	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.set_bnbBurn, **to_local(locals()))

    # 获取BNB抵扣开关状态 (USER_DATA)
    def get_bnbBurn(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bnb-user_data-4

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_bnbBurn, **to_local(locals()))

    # 获取杠杆利率历史 (USER_DATA)
    def get_interestRateHistory(self, asset: str = '', vipLevel: int = '', startTime: int = '', endTime: int = '',
                                recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-42

        Name      	Type	Mandatory	Description
        asset     	str 	YES
        vipLevel  	int 	NO       	默认用户当前等级
        startTime 	int 	NO       	默认7天前
        endTime   	int 	NO       	默认当天，时间间隔最大为1个月
        recvWindow	int 	NO       	赋值不能大于 60000
        '''
        return self.send_request(*self.endpoints.get_interestRateHistory, **to_local(locals()))

    # 获取全仓杠杆利率及限额 (USER_DATA)
    def get_crossMarginData(self, vipLevel: int = '', coin: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-43

        Name      	Type	Mandatory	Description
        vipLevel  	int 	NO       	默认为用户当前VIP等级
        coin      	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_crossMarginData, **to_local(locals()))

    # 获取逐仓杠杆利率及限额 (USER_DATA)
    def get_isolatedMarginData(self, vipLevel: int = '', symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-44

        Name      	Type	Mandatory	Description
        vipLevel  	int 	NO       	默认为用户当前VIP等级
        symbol    	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_isolatedMarginData, **to_local(locals()))

    # 获取逐仓档位信息 (USER_DATA)
    def get_isolatedMarginTier(self, symbol: str = '', tier: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-45

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        tier      	int 	NO       	不传则返回所有逐仓杠杆档位
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_isolatedMarginTier, **to_local(locals()))

    # 查询目前杠杆账户下单数 (TRADE)
    def get_rateLimit_order(self, isIsolated: str = '', symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-13

        Name      	Type	Mandatory	Description
        isIsolated	str 	NO       	是否逐仓杠杆，"TRUE", "FALSE", 默认 "FALSE"
        symbol    	str 	NO       	逐仓交易对，查询逐仓杠杆账户必需
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_rateLimit_order, **to_local(locals()))

    # 杠杆小额资产转换BNB历史 (USER_DATA)
    def get_dribblet(self, startTime: int = '', endTime: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bnb-user_data-5

        Name      	Type	Mandatory	Description
        startTime 	int 	NO
        endTime   	int 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_dribblet, **to_local(locals()))



'''
欧式期权
https://binance-docs.github.io/apidocs/voptions/cn/#185368440e
'''

from .eo_market import EOMarket  # 行情信息
from .eo_account_trade import EOAccountTrade  # 账户与交易
from .eo_market_maker import EOMarketMaker  # 市场接口

Market = EOMarket
AccountTrade = EOAccountTrade
MarketMaker = EOMarketMaker

class EO():
    def __init__(self, key='', secret=''):
        self.market = Market(key=key, secret=secret)
        self.accountTrade = AccountTrade(key=key, secret=secret)
        self.marketMaker = MarketMaker(key=key, secret=secret)



from paux.param import to_local
from pbinance.client import Client
from typing import Union


class _BlvtEndpoints():
    get_tokenInfo = ['https://api.binance.com/','GET', '/sapi/v1/blvt/tokenInfo', False]  # 杠杆代币信息 (MARKET_DATA)
    set_subscribe = ['https://api.binance.com/','POST', '/sapi/v1/blvt/subscribe', True]  # 申购代币 (USER_DATA)
    get_subscribe_record = ['https://api.binance.com/','GET', '/sapi/v1/blvt/subscribe/record', True]  # 查询申购记录 (USER_DATA)
    set_redeem = ['https://api.binance.com/','POST', '/sapi/v1/blvt/redeem', True]  # 赎回代币 (USER_DATA)
    get_redeem_record = ['https://api.binance.com/','GET', '/sapi/v1/blvt/redeem/record', True]  # 查询赎回记录 (USER_DATA)
    get_userLimit = ['https://api.binance.com/','GET', '/sapi/v1/blvt/userLimit', True]  # 查询用户每日申购赎回限额 (USER_DATA)


class Blvt(Client):
    endpoints = _BlvtEndpoints

    # 杠杆代币信息 (MARKET_DATA)
    def get_tokenInfo(self, tokenName: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-10

        Name     	Type	Mandatory	Description
        tokenName	str 	NO       	BTCDOWN, BTCUP
        '''
        return self.send_request(*self.endpoints.get_tokenInfo, **to_local(locals()))

    # 申购代币 (USER_DATA)
    def set_subscribe(self, tokenName: str = '', cost: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-88

        Name      	Type            	Mandatory	Description
        tokenName 	str             	YES      	BTCDOWN, BTCUP
        cost      	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_subscribe, **to_local(locals()))

    # 查询申购记录 (USER_DATA)
    def get_subscribe_record(self, tokenName: str = '', id: int = '', startTime: int = '', endTime: int = '',
                             limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-89

        Name      	Type	Mandatory	Description
        tokenName 	str 	NO       	BTCDOWN, BTCUP
        id        	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 1000, 最大 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_subscribe_record, **to_local(locals()))

    # 赎回代币 (USER_DATA)
    def set_redeem(self, tokenName: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-90

        Name      	Type            	Mandatory	Description
        tokenName 	str             	YES      	BTCDOWN, BTCUP
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_redeem, **to_local(locals()))

    # 查询赎回记录 (USER_DATA)
    def get_redeem_record(self, tokenName: str = '', id: int = '', startTime: int = '', endTime: int = '',
                          limit: int = '',
                          recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-91

        Name      	Type	Mandatory	Description
        tokenName 	str 	NO       	BTCDOWN, BTCUP
        id        	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 1000, 最大 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_redeem_record, **to_local(locals()))

    # 查询用户每日申购赎回限额 (USER_DATA)
    def get_userLimit(self, tokenName: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-92

        Name      	Type	Mandatory	Description
        tokenName 	str 	NO       	BTCDOWN, BTCUP
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_userLimit, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client


class _C2CEndpoints():
    get_orderMatch_listUserOrderHistory = ['https://api.binance.com/','GET', '/sapi/v1/c2c/orderMatch/listUserOrderHistory', True]  # 获取 C2C 交易历史记录 (USER_DATA)


class C2c(Client):
    endpoints = _C2CEndpoints

    # 获取 C2C 交易历史记录 (USER_DATA)
    def get_orderMatch_listUserOrderHistory(self, tradeType: str = '', startTimestamp: int = '', endTimestamp: int = '',
                                            page: int = '', rows: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#c2c-user_data

        Name          	Type	Mandatory	Description
        tradeType     	str 	YES      	BUY, SEll
        startTimestamp	int 	NO
        endTimestamp  	int 	NO
        page          	int 	NO       	default 1
        rows          	int 	NO       	default 100, max 100
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_orderMatch_listUserOrderHistory, **to_local(locals()))



from pbinance import Binance, CM
from pprint import pprint

if __name__ == '__main__':
    # 实例化CM模块
    cm = CM(key='****', secret='****')  # 等同于：cm = Binance( key='****', secret='****').cm
    # 查询全部当前挂单
    result = cm.accountTrade.get_openOrders()
    pprint(result)



# 分类模块
from pbinance import spot  # 币币
from pbinance import cm_future  # 币本位合约
from pbinance import um_future  # U本位合约
from pbinance import european_options  # 欧式期权
from pbinance import margin  # 币币杠杆
from pbinance import other  # 其他
# 类对象
from pbinance.spot import SPOT
from pbinance.cm_future import CM
from pbinance.um_future import UM
from pbinance.european_options import EO
from pbinance.margin import Margin
from pbinance.other import Other

# 别名
cm = cm_future
um = um_future
eo = european_options


class Binance():
    def __init__(self, key='', secret=''):
        self.spot = SPOT(key=key, secret=secret)
        self.cm = CM(key=key, secret=secret)
        self.um = UM(key=key, secret=secret)
        self.eo = EO(key=key, secret=secret)
        self.margin = Margin(key=key, secret=secret)
        self.other = Other(key=key, secret=secret)


__version__ = '1.0.6'



from pbinance import Binance, Other
from pprint import pprint

if __name__ == '__main__':
    # 实例化other模块
    other = Other(key='****', secret='****')  # 等同于：other = Binance( key='****', secret='****').other
    # 获取混合保证金调整质押率历史

    result = other.future.get_loan_adjustCollateral_history()
    pprint(result)



'''
其他接口
https://binance-docs.github.io/apidocs/spot/cn/#185368440e
'''
from .sub_account import SubAccount  # 子母账户
from .savings import Savings  # 币安宝
from .staking import Staking  # Staking
from .mining import Mining  # 矿池
from .futures import Future  # 合约
from .futures_algo import FuturesAlgo  # 合约策略交易
from .blvt import Blvt  # 杠杆代币
from .bs_wap import Bswap  # 币安挖矿
from .fiat import Fiat  # 法币
from .c2c import C2c  # C2C
from .vip_loans import VIPLoans  # VIP借币
from .crypto_loans import CryptoLoans  # 质押借币
from .pay import Pay  # Pay
from .convert import Convert  # 闪兑
from .rebate import Rebate  # 返佣
from .nft import Nft  # NFT
from .giftcard import GiftCard  # 币安码


class Other():
    def __init__(self, key='', secret=''):
        self.subAccount = SubAccount(key=key, secret=secret)
        self.savings = Savings(key=key, secret=secret)
        self.staking = Staking(key=key, secret=secret)
        self.mining = Mining(key=key, secret=secret)
        self.future = Future(key=key, secret=secret)
        self.futuresAlgo = FuturesAlgo(key=key, secret=secret)
        self.blvt = Blvt(key=key, secret=secret)
        self.bswap = Bswap(key=key, secret=secret)
        self.fiat = Fiat(key=key, secret=secret)
        self.c2c = C2c(key=key, secret=secret)
        self.vipLoans = VIPLoans(key=key, secret=secret)
        self.cryptoLoans = CryptoLoans(key=key, secret=secret)
        self.pay = Pay(key=key, secret=secret)
        self.convert = Convert(key=key, secret=secret)
        self.rebate = Rebate(key=key, secret=secret)
        self.nft = Nft(key=key, secret=secret)
        self.giftCard = GiftCard(key=key, secret=secret)



from paux.param import to_local
from pbinance.client import Client


# 法币
class _FiatEndpoints():
    get_orders = ['https://api.binance.com/','GET', '/sapi/v1/fiat/orders', True]  # 获取法币充值/提现历史记录 (USER_DATA)
    get_payments = ['https://api.binance.com/','GET', '/sapi/v1/fiat/payments', True]  # 获取法币支付历史记录 (USER_DATA)


class Fiat(Client):
    endpoints = _FiatEndpoints

    # 获取法币充值/提现历史记录 (USER_DATA)
    def get_orders(self, transactionType: str = '', beginTime: int = '', endTime: int = '', page: int = '',
                   rows: int = '',
                   recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-102

        Name           	Type	Mandatory	Description
        transactionType	str 	YES      	0-deposit,1-withdraw
        beginTime      	int 	NO
        endTime        	int 	NO
        page           	int 	NO       	默认 1
        rows           	int 	NO       	默认 100, 最大 500
        recvWindow     	int 	NO
        '''
        return self.send_request(*self.endpoints.get_orders, **to_local(locals()))

    # 获取法币支付历史记录 (USER_DATA)
    def get_payments(self, transactionType: str = '', beginTime: int = '', endTime: int = '', page: int = '',
                     rows: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-103

        Name           	Type	Mandatory	Description
        transactionType	str 	YES      	0-buy,1-sell
        beginTime      	int 	NO
        endTime        	int 	NO
        page           	int 	NO       	默认 1
        rows           	int 	NO       	默认 100, 最大 500
        recvWindow     	int 	NO
        '''
        return self.send_request(*self.endpoints.get_payments, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union

# 账户与交易
class _SPOTAccountTradeEndpoints():
    set_order_test = ['https://api.binance.com/','POST', '/api/v3/order/test', True]  # 测试下单 (TRADE)
    set_order = ['https://api.binance.com/','POST', '/api/v3/order', True]  # 下单 (TRADE)
    cancel_order = ['https://api.binance.com/','DELETE', '/api/v3/order', True]  # 撤销订单 (TRADE)
    cancel_openOrders = ['https://api.binance.com/','DELETE', '/api/v3/openOrders', False]  # 撤销单一交易对的所有挂单 (TRADE)
    order_cancelReplace = ['https://api.binance.com/','POST', '/api/v3/order/cancelReplace', False]  # 撤消挂单再下单 (TRADE)
    get_order = ['https://api.binance.com/','GET', '/api/v3/order', True]  # 查询订单 (USER_DATA)
    get_openOrders = ['https://api.binance.com/','GET', '/api/v3/openOrders', True]  # 当前挂单 (USER_DATA)
    get_allOrders = ['https://api.binance.com/','GET', '/api/v3/allOrders', True]  # 查询所有订单 (USER_DATA)
    set_order_oco = ['https://api.binance.com/','POST', '/api/v3/order/oco', True]  # OCO下单(TRADE)
    cancel_orderList = ['https://api.binance.com/','DELETE', '/api/v3/orderList', True]  # 取消 OCO 订单(TRADE)
    get_orderList = ['https://api.binance.com/','GET', '/api/v3/orderList', True]  # 查询 OCO (USER_DATA)
    get_allOrderList = ['https://api.binance.com/','GET', '/api/v3/allOrderList', True]  # 查询所有 OCO (USER_DATA)
    get_openOrderList = ['https://api.binance.com/','GET', '/api/v3/openOrderList', True]  # 查询 OCO 挂单 (USER_DATA)
    get_account = ['https://api.binance.com/','GET', '/api/v3/account', True]  # 账户信息 (USER_DATA)
    get_myTrades = ['https://api.binance.com/','GET', '/api/v3/myTrades', True]  # 账户成交历史 (USER_DATA)
    get_rateLimit_order = ['https://api.binance.com/','GET', '/api/v3/rateLimit/order', False]  # 查询目前下单数 (TRADE)


class SPOTAccountTrade(Client):
    endpoints = _SPOTAccountTradeEndpoints

    # 测试下单 (TRADE)
    def set_order_test(self, symbol: str = '', side: str = '', type: str = '', timeInForce: str = '',
                       quantity: Union[float, int] = '', quoteOrderQty: Union[float, int] = '',
                       price: Union[float, int] = '', newClientOrderId: str = '', stopPrice: Union[float, int] = '',
                       trailingDelta: int = '', icebergQty: Union[float, int] = '', newOrderRespType: str = '',
                       strategyId: int = '', strategyType: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-2

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES
        side            	str             	YES      	详见枚举定义：订单方向
        type            	str             	YES      	详见枚举定义：订单类型
        timeInForce     	str             	NO       	详见枚举定义：有效方式
        quantity        	Union[float,int]	NO
        quoteOrderQty   	Union[float,int]	NO
        price           	Union[float,int]	NO
        newClientOrderId	str             	NO       	客户自定义的唯一订单ID。 如果未发送，则自动生成
        stopPrice       	Union[float,int]	NO
        trailingDelta   	int             	NO
        icebergQty      	Union[float,int]	NO
        newOrderRespType	str             	NO       	设置响应JSON。 ACK，RESULT或FULL； "MARKET"和" LIMIT"订单类型默认为"FULL"，所有其他订单默认为"ACK"。
        strategyId      	int             	NO
        strategyType    	int             	NO
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order_test, **to_local(locals()))

    # 下单  (TRADE)
    def set_order(self, symbol: str = '', side: str = '', type: str = '', timeInForce: str = '',
                  quantity: Union[float, int] = '', quoteOrderQty: Union[float, int] = '',
                  price: Union[float, int] = '', newClientOrderId: str = '', stopPrice: Union[float, int] = '',
                  trailingDelta: int = '', icebergQty: Union[float, int] = '', newOrderRespType: str = '',
                  strategyId: int = '', strategyType: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-3

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES
        side            	str             	YES      	详见枚举定义：订单方向
        type            	str             	YES      	详见枚举定义：订单类型
        timeInForce     	str             	NO       	详见枚举定义：有效方式
        quantity        	Union[float,int]	NO
        quoteOrderQty   	Union[float,int]	NO
        price           	Union[float,int]	NO
        newClientOrderId	str             	NO       	客户自定义的唯一订单ID。 如果未发送，则自动生成
        stopPrice       	Union[float,int]	NO
        trailingDelta   	int             	NO
        icebergQty      	Union[float,int]	NO
        newOrderRespType	str             	NO       	设置响应JSON。 ACK，RESULT或FULL； "MARKET"和" LIMIT"订单类型默认为"FULL"，所有其他订单默认为"ACK"。
        strategyId      	int             	NO
        strategyType    	int             	NO
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order, **to_local(locals()))

    # 撤销订单 (TRADE)
    def cancel_order(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', newClientOrderId: str = '',
                     recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-4

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        orderId          	int 	NO
        origClientOrderId	str 	NO
        newClientOrderId 	str 	NO       	用户自定义的本次撤销操作的ID(注意不是被撤销的订单的自定义ID)。如无指定会自动赋值。
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_order, **to_local(locals()))

    # 撤销单一交易对的所有挂单 (TRADE)
    def cancel_openOrders(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-5

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_openOrders, **to_local(locals()))

    # 撤消挂单再下单 (TRADE)
    def order_cancelReplace(self, symbol: str = '', side: str = '', type: str = '', cancelReplaceMode: str = '',
                            timeInForce: str = '', quantity: Union[float, int] = '',
                            quoteOrderQty: Union[float, int] = '', price: Union[float, int] = '',
                            cancelNewClientOrderId: str = '', cancelOrigClientOrderId: str = '',
                            cancelOrderId: int = '', newClientOrderId: str = '', strategyId: int = '',
                            strategyType: int = '', stopPrice: Union[float, int] = '', trailingDelta: int = '',
                            icebergQty: Union[float, int] = '', newOrderRespType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-6

        Name                   	Type            	Mandatory	Description
        symbol                 	str             	YES
        side                   	str             	YES
        type                   	str             	YES
        cancelReplaceMode      	str             	YES
        timeInForce            	str             	NO
        quantity               	Union[float,int]	NO
        quoteOrderQty          	Union[float,int]	NO
        price                  	Union[float,int]	NO
        cancelNewClientOrderId 	str             	NO       	用户自定义的id，如空缺系统会自动赋值
        cancelOrigClientOrderId	str             	NO
        cancelOrderId          	int             	NO
        newClientOrderId       	str             	NO       	用于辨识新订单。
        strategyId             	int             	NO
        strategyType           	int             	NO
        stopPrice              	Union[float,int]	NO
        trailingDelta          	int             	NO
        icebergQty             	Union[float,int]	NO
        newOrderRespType       	str             	NO
        recvWindow             	int             	NO
        '''
        return self.send_request(*self.endpoints.order_cancelReplace, **to_local(locals()))

    # 查询订单 (USER_DATA)
    def get_order(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-20

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        orderId          	int 	NO
        origClientOrderId	str 	NO
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_order, **to_local(locals()))

    # 当前挂单 (USER_DATA)
    def get_openOrders(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-21

        Name      	Type	Mandatory	Description
        symbol    	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrders, **to_local(locals()))

    # 查询所有订单 (USER_DATA)
    def get_allOrders(self, symbol: str = '', orderId: int = '', startTime: int = '', endTime: int = '',
                      limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-22

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        orderId   	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 500; 最大 1000.
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_allOrders, **to_local(locals()))

    # OCO下单(TRADE)
    def set_order_oco(self, symbol: str = '', listClientOrderId: str = '', side: str = '',
                      quantity: Union[float, int] = '', limitClientOrderId: str = '', limitStrategyId: int = '',
                      limitStrategyType: int = '', price: Union[float, int] = '',
                      limitIcebergQty: Union[float, int] = '', trailingDelta: int = '', stopClientOrderId: str = '',
                      stopPrice: Union[float, int] = '', stopStrategyId: int = '', stopStrategyType: int = '',
                      stopLimitPrice: Union[float, int] = '', stopIcebergQty: Union[float, int] = '',
                      stopLimitTimeInForce: str = '', newOrderRespType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-trade

        Name                	Type            	Mandatory	Description
        symbol              	str             	YES
        listClientOrderId   	str             	NO       	整个orderList的唯一ID
        side                	str             	YES      	详见枚举定义：订单方向
        quantity            	Union[float,int]	YES
        limitClientOrderId  	str             	NO       	限价单的唯一ID
        limitStrategyId     	int             	NO
        limitStrategyType   	int             	NO
        price               	Union[float,int]	YES
        limitIcebergQty     	Union[float,int]	NO
        trailingDelta       	int             	NO
        stopClientOrderId   	str             	NO       	止损/止损限价单的唯一ID
        stopPrice           	Union[float,int]	YES
        stopStrategyId      	int             	NO
        stopStrategyType    	int             	NO
        stopLimitPrice      	Union[float,int]	NO
        stopIcebergQty      	Union[float,int]	NO
        stopLimitTimeInForce	str             	NO
        newOrderRespType    	str             	NO       	详见枚举定义：订单返回类型
        recvWindow          	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order_oco, **to_local(locals()))

    # 取消 OCO 订单(TRADE)
    def cancel_orderList(self, symbol: str = '', orderListId: int = '', listClientOrderId: str = '',
                         newClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-trade-2

        Name             	Type	Mandatory	Description
        symbol           	str 	YES
        orderListId      	int 	NO
        listClientOrderId	str 	NO
        newClientOrderId 	str 	NO       	用户自定义的本次撤销操作的ID(注意不是被撤销的订单的自定义ID)。如无指定会自动赋值。
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_orderList, **to_local(locals()))

    # 查询 OCO (USER_DATA)
    def get_orderList(self, orderListId: int = '', origClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data

        Name             	Type	Mandatory	Description
        orderListId      	int 	NO
        origClientOrderId	str 	NO
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_orderList, **to_local(locals()))

    # 查询所有 OCO (USER_DATA)
    def get_allOrderList(self, fromId: int = '', startTime: int = '', endTime: int = '', limit: int = '',
                         recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data-2

        Name      	Type	Mandatory	Description
        fromId    	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认值: 500; 最大值: 1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_allOrderList, **to_local(locals()))

    # 查询 OCO 挂单 (USER_DATA)
    def get_openOrderList(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#oco-user_data-3

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrderList, **to_local(locals()))

    # 账户信息 (USER_DATA)
    def get_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-23

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account, **to_local(locals()))

    # 账户成交历史 (USER_DATA)
    def get_myTrades(self, symbol: str = '', orderId: int = '', startTime: int = '', endTime: int = '',
                     fromId: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-24

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        orderId   	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        fromId    	int 	NO       	起始Trade id。 默认获取最新交易。
        limit     	int 	NO       	默认 500; 最大 1000.
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_myTrades, **to_local(locals()))

    # 查询目前下单数 (TRADE)
    def get_rateLimit_order(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-7

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_rateLimit_order, **to_local(locals()))



from pbinance import Binance
from pprint import pprint

if __name__ == '__main__':
    # 实例化Binance
    # 如果仅获取行情信息不需要key和secret，与账户交易相关的功能需要填写key和secret
    binance = Binance(
        key='****',
        secret='****',
    )
    # spot表示现货 get_ticker_bookTicker获取最优挂单价格
    result = binance.spot.market.get_ticker_bookTicker(
        symbol='BTCUSDT'
    )
    pprint(result)
    # 获取BTCUSDT的最新成交价
    result = binance.spot.market.get_ticker_price(
        symbol='BTCUSDT'
    )
    pprint(result)
    # 账户与交易功能必须填写key和secret
    # 限价单：以2美元单价购买5个DOTUSDT
    result = binance.spot.accountTrade.set_order(
        symbol='DOTUSDT',  # 产品名称
        side='BUY',  # 订单方向
        type='LIMIT',  # 订单类型
        price='2',  # 价格
        quantity='5',  # 数量
        timeInForce='GTC',  # 订单的有效方式
    )
    pprint(result)




'''
币币杠杆(除账户与交易外，币币杠杆交易中很多接口与SPOT公用)
https://binance-docs.github.io/apidocs/spot/cn/#185368440e
'''

from .margin_account_trade import MarginAccountTrade  # 账户与交易

AccountTrade = MarginAccountTrade


class Margin():
    def __init__(self, key='', secret=''):
        from pbinance.spot.spot_market import SPOTMarket as Market
        from pbinance.spot.spot_portfolio_margin import SPOTPortfolioMargin as PortfolioMargin
        from pbinance.spot.wallet import Wallet

        self.accountTrade = AccountTrade(key=key, secret=secret)  # 币币杠杆账户与交易
        # *******************************************************
        self.market = Market(key=key, secret=secret)  # 使用币币交易行情信息
        self.portfolioMargin = PortfolioMargin(key=key, secret=secret)  # 使用币币交易的统一账户
        self.wallet = Wallet(key=key, secret=secret)  # 钱包



'''
U本位合约
https://binance-docs.github.io/apidocs/futures/cn/#185368440e
'''

from .um_market import UMMarket  # 行情信息
from .um_account_trade import UMAccountTrade  # 账户与交易
from .um_portfolio_margin import UMPortfolioMargin  # 统一账户

Market = UMMarket
AccountTrade = UMAccountTrade
PortfolioMargin = UMPortfolioMargin


class UM():
    def __init__(self, key='', secret=''):
        self.market = Market(key=key, secret=secret)
        self.accountTrade = AccountTrade(key=key, secret=secret)
        self.portfolioMargin = PortfolioMargin(key=key, secret=secret)



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# NFT
class _NFTEndpoints():
    get_history_transactions = ['https://api.binance.com/', 'GET', '/sapi/v1/nft/history/transactions', True]  # 获取 NFT 资金流水记录 (USER_DATA)
    get_history_deposit = ['https://api.binance.com/', 'GET', '/sapi/v1/nft/history/deposit', True]  # 获取 NFT 充值记录 (USER_DATA)
    get_history_withdraw = ['https://api.binance.com/', 'GET', '/sapi/v1/nft/history/withdraw', True]  # 获取 NFT 提现记录 (USER_DATA)
    get_user_getAsset = ['https://api.binance.com/', 'GET', '/sapi/v1/nft/user/getAsset', True]  # 获取 NFT 资产 (USER_DATA)


class Nft(Client):
    endpoints = _NFTEndpoints

    # 获取 NFT 资金流水记录 (USER_DATA)
    def get_history_transactions(self, orderType: int = '', startTime: int = '', endTime: int = '', limit: int = '',
                                 page: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#nft-user_data

        Name      	Type	Mandatory	Description
        orderType 	int 	YES      	0:买单，1:卖单，2:版税收入，3:一级市场买单，4:mint 费用
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 50, 最大 50
        page      	int 	NO       	默认 1
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_history_transactions, **to_local(locals()))

    # 获取 NFT 充值记录 (USER_DATA)
    def get_history_deposit(self, startTime: int = '', endTime: int = '', limit: int = '', page: int = '',
                            recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#nft-user_data-2

        Name      	Type	Mandatory	Description
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 50, 最大 50
        page      	int 	NO       	默认 1
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_history_deposit, **to_local(locals()))

    # 获取 NFT 提现记录 (USER_DATA)
    def get_history_withdraw(self, startTime: int = '', endTime: int = '', limit: int = '', page: int = '',
                             recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#nft-user_data-3

        Name      	Type	Mandatory	Description
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 50, 最大 50
        page      	int 	NO       	默认 1
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_history_withdraw, **to_local(locals()))

    # 获取 NFT 资产 (USER_DATA)
    def get_user_getAsset(self, limit: int = '', page: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#nft-user_data-4

        Name      	Type	Mandatory	Description
        limit     	int 	NO       	默认 50, 最大 50
        page      	int 	NO       	默认 1
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_user_getAsset, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union

# 行情信息
class _EOMarketEndpoints():
    get_ping = ['https://eapi.binance.com/','GET', '/eapi/v1/ping', False]  # 测试服务器连通性 PING
    get_time = ['https://eapi.binance.com/','GET', '/eapi/v1/time', False]  # 获取服务器时间
    get_exchangeInfo = ['https://eapi.binance.com/','GET', '/eapi/v1/exchangeInfo', False]  # 获取交易规则和交易对
    get_depth = ['https://eapi.binance.com/','GET', '/eapi/v1/depth', False]  # 深度信息
    get_trades = ['https://eapi.binance.com/','GET', '/eapi/v1/trades', False]  # 近期成交
    get_historicalTrades = ['https://eapi.binance.com/','GET', '/eapi/v1/historicalTrades', False]  # 查询历史成交(MARKET_DATA)
    get_klines = ['https://eapi.binance.com/','GET', '/eapi/v1/klines', False]  # K线数据
    get_mark = ['https://eapi.binance.com/','GET', '/eapi/v1/mark', False]  # 查询期权标记价格
    get_ticker = ['https://eapi.binance.com/','GET', '/eapi/v1/ticker', False]  # 24hr价格变动情况
    get_index = ['https://eapi.binance.com/','GET', '/eapi/v1/index', False]  # 标的最新价格
    get_exerciseHistory = ['https://eapi.binance.com/','GET', '/eapi/v1/exerciseHistory', False]  # 历史行权记录
    get_openInterest = ['https://eapi.binance.com/','GET', '/eapi/v1/openInterest', False]  # 合约持仓量

class EOMarket(Client):
    endpoints = _EOMarketEndpoints

    # 测试服务器连通性 PING
    def get_ping(self):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#ping
        '''
        return self.send_request(*self.endpoints.get_ping, **to_local(locals()))

    # 获取服务器时间
    def get_time(self):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#3f1907847c
        '''
        return self.send_request(*self.endpoints.get_time, **to_local(locals()))

    # 获取交易规则和交易对
    def get_exchangeInfo(self):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#0f3f2d5ee7
        '''
        return self.send_request(*self.endpoints.get_exchangeInfo, **to_local(locals()))

    # 深度信息
    def get_depth(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#38a975b802

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认 100; 可选值:[10, 20, 50, 100, 500, 1000]
        '''
        return self.send_request(*self.endpoints.get_depth, **to_local(locals()))

    # 近期成交
    def get_trades(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#404aacd9b3

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认:100，最大500
        '''
        return self.send_request(*self.endpoints.get_trades, **to_local(locals()))

    # 查询历史成交(MARKET_DATA)
    def get_historicalTrades(self, symbol: str = '', limit: int = '', fromId: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#market_data

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认值:100 最大值:500.
        fromId	int 	NO       	从哪一条成交id开始返回. 缺省返回最近的成交记录
        '''
        return self.send_request(*self.endpoints.get_historicalTrades, **to_local(locals()))

    # K线数据
    def get_klines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#k

        Name     	Type	Mandatory	Description
        symbol   	str 	YES      	交易对
        interval 	str 	YES      	时间间隔
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:500 最大值:1500.
        '''
        return self.send_request(*self.endpoints.get_klines, **to_local(locals()))

    # 查询期权标记价格
    def get_mark(self, symbol: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#d49aa3823d

        Name     	Type	Mandatory	Description
        symbol   	str 	NO       	交易对
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:100 最大值:1000
        '''
        return self.send_request(*self.endpoints.get_mark, **to_local(locals()))

    # 24hr价格变动情况
    def get_ticker(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#24hr

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        '''
        return self.send_request(*self.endpoints.get_ticker, **to_local(locals()))

    # 标的最新价格
    def get_index(self, underlying: str = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#2b0691658e

        Name      	Type	Mandatory	Description
        underlying	str 	YES      	现货交易对如BTCUSDT
        '''
        return self.send_request(*self.endpoints.get_index, **to_local(locals()))

    # 历史行权记录
    def get_exerciseHistory(self, startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#c7c743e6ce

        Name     	Type	Mandatory	Description
        startTime	int 	NO       	开始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:100 最大值:100.
        '''
        return self.send_request(*self.endpoints.get_exerciseHistory, **to_local(locals()))

    # 合约持仓量
    def get_openInterest(self, underlyingAsset: str = '', expiration: str = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#c1c9193984

        Name           	Type	Mandatory	Description
        underlyingAsset	str 	YES      	标的资产，如ETH或BTC
        expiration     	str 	YES      	到期日，如221225
        '''
        return self.send_request(*self.endpoints.get_openInterest, **to_local(locals()))




'''
币币交易
https://binance-docs.github.io/apidocs/spot/cn/#185368440e
'''

from .spot_market import SPOTMarket  # 行情信息
from .spot_account_trade import SPOTAccountTrade  # 账户与交易
from .spot_portfolio_margin import SPOTPortfolioMargin  # 统一账户
from .wallet import Wallet

Market = SPOTMarket
AccountTrade = SPOTAccountTrade
PortfolioMargin = SPOTPortfolioMargin


class SPOT():
    def __init__(self, key='', secret=''):
        self.accountTrade = AccountTrade(key=key, secret=secret)
        self.market = Market(key=key, secret=secret)
        self.portfolioMargin = PortfolioMargin(key=key, secret=secret)
        self.wallet = Wallet(key=key, secret=secret)



from paux.param import to_local
from pbinance.client import Client
from typing import Union


class _BSwapEndpoints():
    get_pools = ['https://api.binance.com/','GET', '/sapi/v1/bswap/pools', False]  # 获取所有流动资金池 (MARKET_DATA)
    get_liquidity = ['https://api.binance.com/','GET', '/sapi/v1/bswap/liquidity', True]  # 获取流动资金池具体信息 (USER_DATA)
    set_liquidityAdd = ['https://api.binance.com/','POST', '/sapi/v1/bswap/liquidityAdd', True]  # 添加流动性 (TRADE)
    set_liquidityRemove = ['https://api.binance.com/','POST', '/sapi/v1/bswap/liquidityRemove', True]  # 移除流动性 (TRADE)
    get_liquidityOps = ['https://api.binance.com/','GET', '/sapi/v1/bswap/liquidityOps', True]  # 获取流动性操作记录 (USER_DATA)
    get_quote = ['https://api.binance.com/','GET', '/sapi/v1/bswap/quote', True]  # 获取报价 (USER_DATA)
    set_swap = ['https://api.binance.com/','POST', '/sapi/v1/bswap/swap', True]  # 交易 (TRADE)
    get_swap = ['https://api.binance.com/','GET', '/sapi/v1/bswap/swap', True]  # 获取交易记录 (USER_DATA)
    get_poolConfigure = ['https://api.binance.com/','GET', '/sapi/v1/bswap/poolConfigure', True]  # 获取币对池的配置信息 (USER_DATA)
    get_addLiquidityPreview = ['https://api.binance.com/','GET', '/sapi/v1/bswap/addLiquidityPreview', True]  # 添加流动性的试算 (USER_DATA)
    get_removeLiquidityPreview = ['https://api.binance.com/','GET', '/sapi/v1/bswap/removeLiquidityPreview', True]  # 移除流动性的试算 (USER_DATA)
    get_unclaimedRewards = ['https://api.binance.com/','GET', '/sapi/v1/bswap/unclaimedRewards', True]  # 查询未领取的奖励数量 (USER_DATA)
    set_claimRewards = ['https://api.binance.com/','POST', '/sapi/v1/bswap/claimRewards', True]  # 领取奖励 (TRADE)
    set_claimedHistory = ['https://api.binance.com/','GET', '/sapi/v1/bswap/claimedHistory', True]  # 获取已领取奖励记录 (USER_DATA)


class Bswap(Client):
    endpoints = _BSwapEndpoints

    # 获取所有流动资金池 (MARKET_DATA)
    def get_pools(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-11
        '''
        return self.send_request(*self.endpoints.get_pools, **to_local(locals()))

    # 获取流动资金池具体信息 (USER_DATA)
    def get_liquidity(self, poolId: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-93

        Name      	Type	Mandatory	Description
        poolId    	int 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_liquidity, **to_local(locals()))

    # 添加流动性 (TRADE)
    def set_liquidityAdd(self, poolId: int = '', type: str = '', asset: str = '', quantity: Union[float, int] = '',
                         recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-15

        Name      	Type            	Mandatory	Description
        poolId    	int             	YES
        type      	str             	NO       	"SINGLE" 为单币添加资产; "COMBINATION" 为双币添加资产。 默认 "SINGLE"
        asset     	str             	YES
        quantity  	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_liquidityAdd, **to_local(locals()))

    # 移除流动性 (TRADE)
    def set_liquidityRemove(self, poolId: int = '', type: str = '', asset: list = '',
                            shareAmount: Union[float, int] = '',
                            recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-16

        Name       	Type            	Mandatory	Description
        poolId     	int             	YES
        type       	str             	YES
        asset      	list            	NO
        shareAmount	Union[float,int]	YES
        recvWindow 	int             	NO
        '''
        return self.send_request(*self.endpoints.set_liquidityRemove, **to_local(locals()))

    # 获取流动性操作记录 (USER_DATA)
    def get_liquidityOps(self, operationId: int = '', poolId: int = '', operation: str = '', startTime: int = '',
                         endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-94

        Name       	Type	Mandatory	Description
        operationId	int 	NO
        poolId     	int 	NO
        operation  	str 	NO
        startTime  	int 	NO
        endTime    	int 	NO
        limit      	int 	NO       	默认 3, 最大 100
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_liquidityOps, **to_local(locals()))

    # 获取报价 (USER_DATA)
    def get_quote(self, quoteAsset: str = '', baseAsset: str = '', quoteQty: Union[float, int] = '',
                  recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-95

        Name      	Type            	Mandatory	Description
        quoteAsset	str             	YES
        baseAsset 	str             	YES
        quoteQty  	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.get_quote, **to_local(locals()))

    # 交易 (TRADE)
    def set_swap(self, quoteAsset: str = '', baseAsset: str = '', quoteQty: Union[float, int] = '',
                 recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-17

        Name      	Type            	Mandatory	Description
        quoteAsset	str             	YES
        baseAsset 	str             	YES
        quoteQty  	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_swap, **to_local(locals()))

    # 获取交易记录 (USER_DATA)
    def get_swap(self, swapId: int = '', startTime: int = '', endTime: int = '', status: int = '', quoteAsset: str = '',
                 baseAsset: str = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-96

        Name      	Type	Mandatory	Description
        swapId    	int 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        status    	int 	NO       	0: 交易中, 1: 交易成功, 2: 交易失败
        quoteAsset	str 	NO
        baseAsset 	str 	NO
        limit     	int 	NO       	默认 3, 最大 100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_swap, **to_local(locals()))

    # 获取币对池的配置信息 (USER_DATA)
    def get_poolConfigure(self, poolId: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-97

        Name      	Type	Mandatory	Description
        poolId    	int 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_poolConfigure, **to_local(locals()))

    # 添加流动性的试算 (USER_DATA)
    def get_addLiquidityPreview(self, poolId: int = '', type: str = '', quoteAsset: str = '',
                                quoteQty: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-98

        Name      	Type            	Mandatory	Description
        poolId    	int             	YES
        type      	str             	YES      	类型为"SINGLE"意思为单币添加;类型为"COMBINATION"意思为双币添加
        quoteAsset	str             	YES
        quoteQty  	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.get_addLiquidityPreview, **to_local(locals()))

    # 移除流动性的试算 (USER_DATA)
    def get_removeLiquidityPreview(self, poolId: int = '', type: str = '', quoteAsset: str = '',
                                   shareAmount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-99

        Name       	Type            	Mandatory	Description
        poolId     	int             	YES
        type       	str             	YES      	类型为"SINGLE"意思为移除获得单币；类型为"COMBINATION"意思为移除获得双币
        quoteAsset 	str             	YES
        shareAmount	Union[float,int]	YES
        recvWindow 	int             	NO
        '''
        return self.send_request(*self.endpoints.get_removeLiquidityPreview, **to_local(locals()))

    # 查询未领取的奖励数量  (USER_DATA)
    def get_unclaimedRewards(self, type: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-100

        Name      	Type	Mandatory	Description
        type      	int 	NO       	0:交易挖矿奖励，1:流动性挖矿奖励，默认为0
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_unclaimedRewards, **to_local(locals()))

    # 领取奖励 (TRADE)
    def set_claimRewards(self, type: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-18

        Name      	Type	Mandatory	Description
        type      	int 	NO       	0:交易挖矿奖励，1:流动性挖矿奖励，默认为0
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_claimRewards, **to_local(locals()))

    # 获取已领取奖励记录 (USER_DATA)
    def set_claimedHistory(self, poolId: int = '', assetRewards: str = '', type: int = '', startTime: int = '',
                           endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-101

        Name        	Type	Mandatory	Description
        poolId      	int 	NO
        assetRewards	str 	NO
        type        	int 	NO       	0:交易挖矿奖励，1:流动性挖矿奖励，默认为0
        startTime   	int 	NO
        endTime     	int 	NO
        limit       	int 	NO       	Default 3, max 100
        recvWindow  	int 	NO
        '''
        return self.send_request(*self.endpoints.set_claimedHistory, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client


# 统一账户
class _SPOTPortfolioMarginEndpoints():
    portfolio_account = ['https://api.binance.com/','GET', '/sapi/v1/portfolio/account', True]  # 查询统一账户信息 (USER_DATA)
    portfolio_collateralRate = ['https://api.binance.com/','GET', '/sapi/v1/portfolio/collateralRate', False]  # 统一账户资产质押率 (MARKET_DATA)
    portfolio_pmLoan = ['https://api.binance.com/','GET', '/sapi/v1/portfolio/pmLoan', False]  # 查询统一账户穿仓借贷金额 (USER_DATA)
    portfolio_repay = ['https://api.binance.com/','POST', '/sapi/v1/portfolio/repay', False]  # 偿还统一账户穿仓负债


class SPOTPortfolioMargin(Client):
    endpoints = _SPOTPortfolioMarginEndpoints

    # 查询统一账户信息 (USER_DATA)
    def portfolio_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-86

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.portfolio_account, **to_local(locals()))

    # 统一账户资产质押率 (MARKET_DATA)
    def portfolio_collateralRate(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-9
        '''
        return self.send_request(*self.endpoints.portfolio_collateralRate, **to_local(locals()))

    # 查询统一账户穿仓借贷金额 (USER_DATA)
    def portfolio_pmLoan(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-87

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.portfolio_pmLoan, **to_local(locals()))

    # 偿还统一账户穿仓负债
    def portfolio_repay(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#eb9fb28dd5

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.portfolio_repay, **to_local(locals()))



import hmac
import hashlib
import requests
import urllib.parse as up
import time


def request_retry_wrapper(retry_num=50, retry_delay=0.1):
    def wrapper1(func):
        def wrapper2(*args, **kwargs):
            for i in range(retry_num - 1):
                result = func(*args, **kwargs)
                if result['code'] == 200:
                    return result
                elif result['code'] in [-1001, -1003, -1004, -1007, -1008, -1016]:
                    time.sleep(retry_delay)
                    continue
                else:
                    return result
            result = func(*args, **kwargs)
            return result

        return wrapper2

    return wrapper1


class Client():
    def __init__(self, key='', secret=''):
        self.key = key
        self.secret = secret
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json;charset=utf-8",
                "User-Agent": "binance-api",
                "X-MBX-APIKEY": key,
            }
        )

    @request_retry_wrapper()
    def send_request(self, api_url, method, path, sign=True, **kwargs):
        params_no_empty = {}
        for key, v in kwargs.items():
            if not (v == '' or v == None):
                params_no_empty[key] = v
        if sign:
            params_no_empty["timestamp"] = int(time.time() * 1000)
            query_string = up.urlencode(params_no_empty, True).replace("%40", "@")
            signature = self._get_sign(query_string)
            params_no_empty["signature"] = signature
        url = up.urljoin(api_url, path)


        response = getattr(self.session, method.lower())(
            url=url,
            params=up.urlencode(params_no_empty, True).replace("%40", "@")
        )
        data = response.json()
        # 如果有code
        if type(data) == dict and 'code' in data.keys():
            result = data
        else:
            result = {
                'code': 200,
                'data': data,
                'msg': '',
            }
        return result

    def _get_sign(self, data):
        m = hmac.new(self.secret.encode("utf-8"), data.encode("utf-8"), hashlib.sha256)
        return m.hexdigest()



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 合约策略
class _FuturesAlgoEndpoints():
    set_newOrderVp = ['https://api.binance.com/','POST', '/sapi/v1/algo/futures/newOrderVp', True]  # 成交量份额参与算法(VP)下单 (TRADE)
    set_newOrderTwap = ['https://api.binance.com/','POST', '/sapi/v1/algo/futures/newOrderTwap', True]  # 时间加权平均价格策略(Twap)下单 (TRADE)
    delete_order = ['https://api.binance.com/','DELETE', '/sapi/v1/algo/futures/order', True]  # 取消策略订单 (TRADE)
    get_openOrders = ['https://api.binance.com/','GET', '/sapi/v1/algo/futures/openOrders', True]  # 查询当前策略订单挂单 (USER_DATA)
    get_historicalOrders = ['https://api.binance.com/','GET', '/sapi/v1/algo/futures/historicalOrders', True]  # 查询历史策略订单 (USER_DATA)
    get_subOrders = ['https://api.binance.com/','GET', '/sapi/v1/algo/futures/subOrders', True]  # 查询执行子订单 (USER_DATA)


class FuturesAlgo(Client):
    endpoints = _FuturesAlgoEndpoints

    # 成交量份额参与算法(VP)下单 (TRADE)
    def set_newOrderVp(self, symbol: str = '', side: str = '', positionSide: str = '', quantity: Union[float, int] = '',
                       urgency: str = '', clientAlgoId: str = '', reduceOnly: bool = '',
                       limitPrice: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#vp-trade

        Name        	Type            	Mandatory	Description
        symbol      	str             	YES      	交易对 eg. BTCUSDT
        side        	str             	YES      	买卖方向 ( BUY or SELL )
        positionSide	str             	NO       	持仓方向，单向持仓模式下非必填，默认且仅可填BOTH;在双向持仓模式下必填,且仅可选择 LONG 或 SHORT
        quantity    	Union[float,int]	YES
        urgency     	str             	YES      	代表当前执行的相对速率; ENUM: LOW（慢）, MEDIUM（中等）, HIGH（快）
        clientAlgoId	str             	NO       	必须传入32位，如果未发送，则自动生成
        reduceOnly  	bool            	NO       	true, false; 非双开模式下默认false；双开模式下不接受此参数； 开仓不接受此参数
        limitPrice  	Union[float,int]	NO       	限价单价格; 若未发送，则以市场价下单
        recvWindow  	int             	NO
        '''
        return self.send_request(*self.endpoints.set_newOrderVp, **to_local(locals()))

    # 时间加权平均价格策略(Twap)下单 (TRADE)
    def set_newOrderTwap(self, symbol: str = '', side: str = '', positionSide: str = '',
                         quantity: Union[float, int] = '', duration: int = '', clientAlgoId: str = '',
                         reduceOnly: bool = '', limitPrice: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#twap-trade

        Name        	Type            	Mandatory	Description
        symbol      	str             	YES      	交易对 eg. BTCUSDT
        side        	str             	YES      	买卖方向 ( BUY or SELL )
        positionSide	str             	NO       	持仓方向，单向持仓模式下非必填，默认且仅可填BOTH;在双向持仓模式下必填,且仅可选择 LONG 或 SHORT
        quantity    	Union[float,int]	YES
        duration    	int             	YES      	请以秒为单位发送[300,86400]；少于 5 分钟 => 默认为 5 分钟；大于 24h => 默认为 24h
        clientAlgoId	str             	NO       	必须传入32位，如果未发送，则自动生成
        reduceOnly  	bool            	NO       	true, false; 非双开模式下默认false；双开模式下不接受此参数； 开仓不接受此参数
        limitPrice  	Union[float,int]	NO       	限价单价格; 若未发送，则以市场价下单
        recvWindow  	int             	NO
        '''
        return self.send_request(*self.endpoints.set_newOrderTwap, **to_local(locals()))

    # 取消策略订单 (TRADE)
    def delete_order(self, algoId: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-14

        Name      	Type	Mandatory	Description
        algoId    	int 	YES      	eg. 14511
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.delete_order, **to_local(locals()))

    # 查询当前策略订单挂单 (USER_DATA)
    def get_openOrders(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-83

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrders, **to_local(locals()))

    # 查询历史策略订单 (USER_DATA)
    def get_historicalOrders(self, symbol: str = '', side: str = '', startTime: int = '', endTime: int = '',
                             page: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-84

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对 eg. BTCUSDT
        side      	str 	NO       	BUY 或者 SELL
        startTime 	int 	NO       	毫秒级时间戳  eg.1641522717552
        endTime   	int 	NO       	毫秒级时间戳  eg.1641522526562
        page      	int 	NO       	默认 1
        pageSize  	int 	NO       	最小 1, 最大 100; 默认 100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_historicalOrders, **to_local(locals()))

    # 查询执行子订单 (USER_DATA)
    def get_subOrders(self, algoId: int = '', page: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-85

        Name      	Type	Mandatory	Description
        algoId    	int 	YES
        page      	int 	NO       	默认1
        pageSize  	int 	NO       	最小 1， 最大 100; 默认 100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_subOrders, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 统一账户
class _CMPortfolioMarginEndpoints():
    get_pmExchangeInfo = ['https://dapi.binance.com/','GET', '/dapi/v1/pmExchangeInfo', False]  # 获取统一账户交易规则
    get_pmAccountInfo = ['https://dapi.binance.com/','GET', '/dapi/v1/pmAccountInfo', False]  # 查询统一账户账户信息 (USER_DATA)


class CMPortfolioMargin(Client):
    endpoints = _CMPortfolioMarginEndpoints

    # 获取统一账户交易规则
    def get_pmExchangeInfo(self, symbol: str = '', pair: str = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#72815f42fe

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        pair  	str 	NO       	标的交易对
        '''
        return self.send_request(*self.endpoints.get_pmExchangeInfo, **to_local(locals()))

    # 查询统一账户账户信息 (USER_DATA)
    def get_pmAccountInfo(self, asset: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-15

        Name      	Type	Mandatory	Description
        asset     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_pmAccountInfo, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 质押借币
class _CryptoLoansEndpoints():
    get_income = ['https://api.binance.com/','GET', '/sapi/v1/loan/income', True]  # 获取质押借币资金流水 (USER_DATA)
    set_borrow = ['https://api.binance.com/','POST', '/sapi/v1/loan/borrow', False]  # 借币 - 质押借币借贷 (TRADE)
    get_borrow_history = ['https://api.binance.com/','GET', '/sapi/v1/loan/borrow/history', False]  # 借币 - 查询质押借币历史记录 (USER_DATA)
    get_ongoing_orders = ['https://api.binance.com/','GET', '/sapi/v1/loan/ongoing/orders', False]  # 借币 - 查询借款中订单列表 (USER_DATA)
    set_repay = ['https://api.binance.com/','POST', '/sapi/v1/loan/repay', False]  # 还款 - 质押借币还款 (TRADE)
    get_repay_history = ['https://api.binance.com/','GET', '/sapi/v1/loan/repay/history', False]  # 还款 - 查询还款记录历史 (USER_DATA)
    set_adjust_ltv = ['https://api.binance.com/','POST', '/sapi/v1/loan/adjust/ltv', False]  # 调整质押率 - 质押借币调整质押率 (TRADE)
    get_ltv_adjustment_history = ['https://api.binance.com/','GET', '/sapi/v1/loan/ltv/adjustment/history', False]  # 调整质押率 - 查询质押率调整历史 (USER_DATA)
    get_loanable_data = ['https://api.binance.com/','GET', '/sapi/v1/loan/loanable/data', False]  # 查询可借币种数据 (USER_DATA)
    get_collateral_data = ['https://api.binance.com/','GET', '/sapi/v1/loan/collateral/data', False]  # 查询抵押币种数据 (USER_DATA)
    get_repay_collateral_rate = ['https://api.binance.com/','GET', '/sapi/v1/loan/repay/collateral/rate', False]  # 查询抵押币种还款汇率 (USER_DATA)
    set_customize_margin_call = ['https://api.binance.com/','POST', '/sapi/v1/loan/customize/margin_call', False]  # 质押借币自定义补仓质押率 (TRADE)


class CryptoLoans(Client):
    endpoints = _CryptoLoansEndpoints

    # 获取质押借币资金流水 (USER_DATA)
    def get_income(self, asset: str = '', type: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                   recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-104

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        type      	str 	NO          默认返回所有类型 枚举值：借入 borrowIn,抵押金使用collateralSpent, 还款金额repayAmount, 抵押物返还collateralReturn, 增加抵押物addCollateral, 减少抵押物removeCollateral, 强平后返还抵押物collateralReturnAfterLiquidation
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认 20, 最大 100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_income, **to_local(locals()))

    # 借币 - 质押借币借贷 (TRADE)
    def set_borrow(self, loanCoin: str = '', loanAmount: Union[float, int] = '', collateralCoin: str = '',
                   collateralAmount: Union[float, int] = '', loanTerm: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-19

        Name            	Type            	Mandatory	Description
        loanCoin        	str             	YES
        loanAmount      	Union[float,int]	NO       	当collateralAmount为空时，需必填
        collateralCoin  	str             	YES
        collateralAmount	Union[float,int]	NO       	当loanAmount为空时，需必填
        loanTerm        	int             	YES      	7/14/30/90/180 天
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_borrow, **to_local(locals()))

    # 借币 - 查询质押借币历史记录 (USER_DATA)
    def get_borrow_history(self, orderId: int = '', loanCoin: str = '', collateralCoin: str = '', startTime: int = '',
                           endTime: int = '', current: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-105

        Name          	Type	Mandatory	Description
        orderId       	int 	NO          POST /sapi/v1/loan/borrow 中的 orderId
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页数，从1开始。默认值：1；最大：1000。
        limit         	int 	NO       	默认值：10；最大：100。
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_borrow_history, **to_local(locals()))

    # 借币 - 查询借款中订单列表 (USER_DATA)
    def get_ongoing_orders(self, orderId: int = '', loanCoin: str = '', collateralCoin: str = '', current: int = '',
                           limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-106

        Name          	Type	Mandatory	Description
        orderId       	int 	NO
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        current       	int 	NO       	当前查询页数，从1开始。默认值：1；最大：1000。
        limit         	int 	NO       	默认值：10；最大：100。
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_ongoing_orders, **to_local(locals()))

    # 还款 - 质押借币还款 (TRADE)
    def set_repay(self, orderId: int = '', amount: Union[float, int] = '', type: int = '', collateralReturn: bool = '',
                  recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-20

        Name            	Type            	Mandatory	Description
        orderId         	int             	YES
        amount          	Union[float,int]	YES
        type            	int             	NO       	默认值：1。 1：用借贷币还款；2：用抵押币还款。
        collateralReturn	bool            	NO       	默认值：TRUE。TRUE：多余的抵押金退回现货钱包；FALSE: 多余的抵押金保留在原订单里。
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_repay, **to_local(locals()))

    # 还款 - 查询还款记录历史 (USER_DATA)
    def get_repay_history(self, orderId: int = '', loanCoin: str = '', collateralCoin: str = '', startTime: int = '',
                          endTime: int = '', current: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-107

        Name          	Type	Mandatory	Description
        orderId       	int 	NO
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页数，从1开始。默认值：1；最大：1000。
        limit         	int 	NO       	默认值：10；最大：100。
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_repay_history, **to_local(locals()))

    # 调整质押率 - 质押借币调整质押率 (TRADE)
    def set_adjust_ltv(self, orderId: int = '', amount: Union[float, int] = '', direction: str = '',
                       recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-21

        Name      	Type            	Mandatory	Description
        orderId   	int             	YES
        amount    	Union[float,int]	YES
        direction 	str             	YES      	"ADDITIONAL", "REDUCED"
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_adjust_ltv, **to_local(locals()))

    # 调整质押率 - 查询质押率调整历史 (USER_DATA)
    def get_ltv_adjustment_history(self, orderId: int = '', loanCoin: str = '', collateralCoin: str = '',
                                   startTime: int = '', endTime: int = '', current: int = '', limit: int = '',
                                   recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-108

        Name          	Type	Mandatory	Description
        orderId       	int 	NO
        loanCoin      	str 	NO
        collateralCoin	str 	NO
        startTime     	int 	NO
        endTime       	int 	NO
        current       	int 	NO       	当前查询页数，从1开始。默认值：1；最大：1000。
        limit         	int 	NO       	默认值：10；最大：100。
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_ltv_adjustment_history, **to_local(locals()))

    # 查询可借币种数据 (USER_DATA)
    def get_loanable_data(self, loanCoin: str = '', vipLevel: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-109

        Name      	Type	Mandatory	Description
        loanCoin  	str 	NO
        vipLevel  	int 	NO       	默认：用户当前VIP登记。如有特殊配置，则填“-1”查询
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_loanable_data, **to_local(locals()))

    # 查询抵押币种数据 (USER_DATA)
    def get_collateral_data(self, collateralCoin: str = '', vipLevel: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-110

        Name          	Type	Mandatory	Description
        collateralCoin	str 	NO
        vipLevel      	int 	NO       	默认：用户当前VIP登记。如有特殊配置，则填“-1”查询
        recvWindow    	int 	NO
        '''
        return self.send_request(*self.endpoints.get_collateral_data, **to_local(locals()))

    # 查询抵押币种还款汇率 (USER_DATA)
    def get_repay_collateral_rate(self, loanCoin: str = '', collateralCoin: str = '',
                                  repayAmount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-111

        Name          	Type            	Mandatory	Description
        loanCoin      	str             	YES
        collateralCoin	str             	YES
        repayAmount   	Union[float,int]	YES      	以借贷币种为单位的还款金额
        recvWindow    	int             	NO
        '''
        return self.send_request(*self.endpoints.get_repay_collateral_rate, **to_local(locals()))

    # 质押借币自定义补仓质押率 (TRADE)
    def set_customize_margin_call(self, orderId: int = '', collateralCoin: str = '', marginCall: Union[float, int] = '',
                                  recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade-22

        Name          	Type            	Mandatory	Description
        orderId       	int             	NO       	当collateralCoin为空时，需必填。orderId或collateralCoin只需传一个，如果两个参数同时传，以orderId为准。
        collateralCoin	str             	NO       	当orderID为空时，需必填。orderId或collateralCoin只需传一个，如果两个参数同时传，以orderId为准。
        marginCall    	Union[float,int]	YES
        recvWindow    	int             	NO
        '''
        return self.send_request(*self.endpoints.set_customize_margin_call, **to_local(locals()))
